import processing.video.*;

CircleTracker outCircle;
CircleTracker inCircle;
Point outPoint;
Point inPoint;
RadialSlit outerSlit;

//motion circle
CircleTracker motionoutCircle;
CircleTracker motioninCircle;
Point motionoutPoint;
Point motioninPoint;
RadialSlit motionouterSlit;


float numSecs = 60;
float densityLeft = 30;
//float densityLeft = 1;

float densityRight = 20;
//this is based on sampling a video that is 240, so the vid_div needs to be a factor of slit Length
float slitLength = 240;
float centerSize = 40;
float circleRadius = slitLength + centerSize;
int vid_div = 1;
float yOffset = 80;

float radRange = 75;
float radCap = 5000;
int radMax = (int)slitLength-1;

int fadeRate = 10;

Capture video;

int videoSliceX;
int stillDrawPositionX;
int moveDrawPositionX;
int[] prevSlit;

int stillThresh = 1000;
int moveThresh = 1000;
int scanX = 0;
boolean scanFwd = true;
int scanMax = 100;

int bgColor = 25;
int leftColor = bgColor;
int rightColor = bgColor;
int maxColor = 120;

float motionEst;

void setup() {
  size(1280,1024);
  frameRate(15);
  background(30,30,30);
  //background(0);
  stroke(255,255,255);
  fill(255,255,255);
  
  noCursor();
  //where to start the x and y points

  //this is right circle
  outPoint = new Point(0,0);
  outCircle = new CircleTracker(width*0.75,height/2-yOffset,width*0.75,height/2-circleRadius-yOffset,numSecs*densityRight,90,circleRadius,1);
  /// inside circle
  inPoint = new Point(0,0);
  inCircle = new CircleTracker(width*0.75,height/2-yOffset,width*0.75,height/2-(circleRadius-slitLength)-yOffset,numSecs*densityRight,90,circleRadius-slitLength,1);
  //slit
  outerSlit = new RadialSlit((int)slitLength);

  //this is the left
  motionoutPoint = new Point(0,0);
  motionoutCircle = new CircleTracker(width*0.25,height/2-yOffset,width*0.25,height/2-circleRadius-yOffset,numSecs*densityLeft,90,circleRadius,0);
  /// inside circle
  motioninPoint = new Point(0,0);
  motioninCircle = new CircleTracker(width*0.25,height/2-yOffset,width*0.25,height/2-(circleRadius-slitLength)-yOffset,numSecs*densityLeft,90,circleRadius-slitLength,0);
  //slit
  motionouterSlit = new RadialSlit((int)slitLength);

  //

  // Uses the default video input, see the reference if this causes an error
  video = new Capture(this, 320, 240, 15);

  videoSliceX = video.width / 2;
  stillDrawPositionX = width - 1;
  moveDrawPositionX = width - 1;
  //background(0);

  prevSlit = new int[video.height];
  for (int y = 0; y < video.height; y++){
    prevSlit[y] = 0;
  }
}

void draw() {  
  //outerSlit.drawPoints();

  if (video.available()) {
    video.read();
    video.loadPixels();

    // Copy a column of pixels from the middle of the video 
    // To a location moving slowly across the canvas.
    loadPixels();

    int diff = 0;
    for (int y = 0; y < video.height; y++){
      int getPixelIndex = y*video.width  + videoSliceX;
      float currB = brightness(video.pixels[getPixelIndex]);
      float prevB = brightness(prevSlit[y]);
      diff += abs(currB - prevB);
      prevSlit[y] = video.pixels[getPixelIndex];
    }
    //println(diff);

    if (diff < stillThresh) {
      //if (true) {
      outPoint = outCircle.getNextPoint();
      inPoint = inCircle.getNextPoint();
      outerSlit.setPoints(outPoint,inPoint);

      for (int i = 0; i < video.height; i+=vid_div){
        int setPixelIndex = i*width + stillDrawPositionX;
        int getPixelIndex = i*video.width  + videoSliceX;
        //pixels[setPixelIndex] = video.pixels[getPixelIndex + scanX - (scanMax / 2)];
        //pixels[setPixelIndex] = video.pixels[getPixelIndex + scanX];
        int index = i / vid_div;
        //set((int)outerSlit.getPoint(index).x, (int)outerSlit.getPoint(index).y, video.pixels[getPixelIndex + scanX]);
        int myX = (int)outerSlit.getPoint(index).x;
        int myY = (int)outerSlit.getPoint(index).y;
        pixels[myY * width + myX] = video.pixels[getPixelIndex + scanX - (scanMax / 2)];
        //println((int)outerSlit.getPoint(index).x);
      }
      if (scanFwd) {
        scanX++;
      } 
      else {
        scanX--;
      }
      if (scanX > scanMax) {
        scanFwd = false;
      }
      if (scanX < 0) {
        scanFwd = true;
      }
      stillDrawPositionX--;
      // Wrap the position back to the beginning if necessary.
      if (stillDrawPositionX < 0) {
        stillDrawPositionX = width - 1;
      }
      updatePixels();
      motioninCircle.fillInside(color(leftColor));
      inCircle.fillInside(color(rightColor));
      leftColor-=fadeRate;
      if (leftColor < bgColor) { 
        leftColor = bgColor;
      }
      rightColor+=fadeRate;
      if (rightColor > maxColor) {
        rightColor = maxColor;
      }
    }

    // 
    if (diff > moveThresh) {

      drawMotionSlit();
      drawMotionSlit();
      drawMotionSlit();
      drawMotionSlit();
      
      updatePixels();
      motioninCircle.fillInside(color(leftColor));
      inCircle.fillInside(color(rightColor));
      rightColor-=fadeRate;
      if (rightColor < bgColor) { 
        rightColor = bgColor;
      }
      leftColor+=fadeRate;
      if (leftColor > maxColor) {
        leftColor = maxColor;
      }
    }

    // draw a bar graph
    motionEst = (motionEst * 4 + diff) / 5.0;
   /*
    fill(bgColor);
    rect(0, height-50, width, 50);
    float barLen = map(motionEst, 0, 10000, 0, width);
    fill(200);
    rect(0, height-50, (int)barLen, 50);
  */

  }
}

void drawMotionSlit() {


  float rad = map(motionEst, moveThresh, radCap, 0, radRange);
  //rad = radMax - rad;
  if (rad > radMax) {
    rad=radMax;
  }

  motionoutPoint = motionoutCircle.getNextPoint();
  motioninPoint = motioninCircle.getNextPoint();
  motionouterSlit.setPoints(motionoutPoint,motioninPoint);
  for (int i = 0; i < video.height; i+=vid_div){
    if (i > rad) {
      int getPixelIndex = i*video.width  + videoSliceX;
      //pixels[setPixelIndex] = video.pixels[getPixelIndex + scanX - (scanMax / 2)];
      //pixels[setPixelIndex] = video.pixels[getPixelIndex + scanX];
      int index = i / vid_div;
      //set((int)outerSlit.getPoint(index).x, (int)outerSlit.getPoint(index).y, video.pixels[getPixelIndex + scanX]);
      int myX = (int)motionouterSlit.getPoint(index).x;
      int myY = (int)motionouterSlit.getPoint(index).y;
      pixels[myY * width + myX] = video.pixels[getPixelIndex + scanX];
      //println((int)outerSlit.getPoint(index).x);
    }
  }
}










